<?php

/*
 * Plugin: Third-party Plugins UpToDate
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * For a detailed description and documentation, please refer to:
 * http://www.undef.name/MPAseco/Third-party-Plugins-UpToDate.php
 *
 * ----------------------------------------------------------------------------------
 * Author:		undef.de
 * Version:		0.9.3
 * Date:		2014-01-05
 * Copyright:		2011 - 2014 by undef.de
 * System:		MPAseco/0.83+
 * Game:		Shootmania (ManiaPlanet)
 * ----------------------------------------------------------------------------------
 *
 * LICENSE: This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * ----------------------------------------------------------------------------------
 *
 * Dependencies: none
 *
 * ManialinkID's
 * ~~~~~~~~~~~~~
 * 91900			id for manialink Window
 * 91901			id for manialink LegendWindow
 *
 * ActionID's
 * ~~~~~~~~~~
 *  91900			id for action open Window
 *  91901			id for action close Window
 *  91902			id for action open LegendWindow
 *  91903			id for action close LegendWindow
 *  91904 - 91910		UNSED BUT RESERVED
 * -91910 to -91960		id for action Pluginslist previous page in Table (max. 50 pages a 20 entries = 1000 total)
 *  91910 to  91960		id for action Pluginslist next page in Table (max. 50 pages a 20 entries = 1000 total)
 *
 */

Aseco::registerEvent('onSync',				'utd_onSync');
Aseco::registerEvent('onPlayerConnect',			'utd_onPlayerConnect');
Aseco::registerEvent('onPlayerManialinkPageAnswer',	'utd_onPlayerManialinkPageAnswer');
Aseco::registerEvent('onEndMap1',			'utd_onEndMap1');

Aseco::addChatCommand('checkplugins',			'Checks for new releases of installed Plugins', true);

global $utd_config;

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

function utd_onSync ($aseco) {
	global $utd_config;


	// Check for the right MPAseco-Version
	$mpaseco_min_version = '0.83';
	if ( defined('MPASECO_VERSION') ) {
		$version = str_replace(
			array('a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i'),
			array('.1','.2','.3','.4','.5','.6','.7','.8','.9'),
			MPASECO_VERSION
		);
		if ( version_compare($version, $mpaseco_min_version, '<') ) {
			trigger_error('[plugin.third_party_plugins_uptodate.php] Not supported MPAseco version ('. MPASECO_VERSION .')! Please update to min. version '. $mpaseco_min_version .'!', E_USER_ERROR);
		}
	}
	else {
		trigger_error('[plugin.third_party_plugins_uptodate.php] Can not identify the System, "MPASECO_VERSION" is unset! This plugin runs only with MPAseco/'. $mpaseco_min_version .' and up.', E_USER_ERROR);
	}

	if ($aseco->server->getGame() != 'MP') {
		trigger_error('[plugin.third_party_plugins_uptodate.php] This plugin supports only SM (ManiaPlanet), can not start with a "'. $aseco->server->getGame() .'" Dedicated-Server!', E_USER_ERROR);
	}

	if (!$utd_config = $aseco->xml_parser->parseXML('configs/plugins/third_party_plugins_uptodate.xml')) {
		trigger_error("[plugin.third_party_plugins_uptodate.php] Could not read/parse config file 'configs/plugins/third_party_plugins_uptodate.xml'!", E_USER_ERROR);
	}
	$utd_config = $utd_config['THIRD_PARTY_PLUGINS_UPTODATE'];

	if ( (isset($utd_config['URL'][0])) && ($utd_config['URL'][0] == '')) {
		trigger_error("[plugin.third_party_plugins_uptodate.php] <url> is empty in third_party_plugins_uptodate.xml!", E_USER_ERROR);
	}

	$utd_config['ENABLE_CHECKSUM_CHECK'][0] = ((strtoupper($utd_config['ENABLE_CHECKSUM_CHECK'][0]) == 'TRUE') ? true : false);

	$utd_config['version'] = '0.9.3';
	$utd_config['manialink_id'] = '919';
	$utd_config['display_state'] = false;
	$utd_config['plugins'] = array();

	$options = array(
		'http'	=> array(
			'method'	=> 'GET',
			'user_agent'	=> 'MPAseco Third-party Plugins UpToDate/'. $utd_config['version']
		),
		'ftp'	=> array(
			'overwrite'	=> true
		)
	);
	$utd_config['stream_context'] = stream_context_create($options);

	// Register this to the global version pool (for up-to-date checks)
	$aseco->plugin_versions[] = array(
		'plugin'	=> 'plugin.third_party_plugins_uptodate.php',
		'author'	=> 'undef.de',
		'version'	=> $utd_config['version']
	);
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

function chat_checkplugins ($aseco, $command) {
	global $utd_config;


	// Check for a MasterAdmin
	if ( $aseco->isMasterAdmin($command['author']) ) {
		if (count($utd_config['plugins']) == 0) {
			// Check for outdated Plugins
			utd_checkForOutdatedPlugins();
		}

		// Always display the PluginlistWindow
		$aseco->client->query('TriggerModeScriptEvent', 'LibXmlRpc_DisableAltMenu', $command['author']->login);
		$xml = utd_buildPluginListWindow(0, true);

		// Send to given Player
		$aseco->client->query('SendDisplayManialinkPageToLogin', $command['author']->login, $xml, 0, false);
	}
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

function utd_onPlayerConnect ($aseco, $player) {
	global $utd_config;


	// Check for a MasterAdmin
	if ( $aseco->isMasterAdmin($player) ) {
		$result = false;
		if (count($utd_config['plugins']) == 0) {
			// Check for outdated Plugins
			$result = utd_checkForOutdatedPlugins();
		}

		if ($result == true) {
			$aseco->client->query('TriggerModeScriptEvent', 'LibXmlRpc_DisableAltMenu', $player->login);
			$xml = utd_buildPluginListWindow(0, true);

			// Send to given Player
			$aseco->client->query('SendDisplayManialinkPageToLogin', $player->login, $xml, 0, false);
		}
	}
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

// $answer = [0]=PlayerUid, [1]=Login, [2]=Answer
function utd_onPlayerManialinkPageAnswer ($aseco, $answer) {
	global $utd_config;


	// If id = 0, bail out immediately
	if ($answer[2] == 0) {
		return;
	}

	// Get the Player object
	$player = $aseco->server->players->player_list[$answer[1]];

	// Check for a MasterAdmin
	if ( !$aseco->isMasterAdmin($player) ) {
		return;
	}


	$xml = '';

	if ($answer[2] == (int)$utd_config['manialink_id'] .'00') {

		if (count($utd_config['plugins']) == 0) {
			// Check for outdated Plugins
			utd_checkForOutdatedPlugins();
		}

		// Open Window
		$aseco->client->query('TriggerModeScriptEvent', 'LibXmlRpc_DisableAltMenu', $player->login);
		$xml .= utd_buildPluginListWindow(0, true);

	}
	else if ($answer[2] == (int)$utd_config['manialink_id'] .'01') {

		// Close all Windows
		$aseco->client->query('TriggerModeScriptEvent', 'LibXmlRpc_EnableAltMenu', $player->login);
		$xml .= utd_closeAllWindows();

	}
	else if ($answer[2] == (int)$utd_config['manialink_id'] .'02') {

		// Open LegendWindow
		$xml .= utd_buildLegendWindow(true);

	}
	else if ($answer[2] == (int)$utd_config['manialink_id'] .'03') {

		// Close LegendWindow
		$xml .= utd_buildLegendWindow(false);

	}
	else if ( ($answer[2] <= -(int)$utd_config['manialink_id'] .'10') && ($answer[2] >= -(int)$utd_config['manialink_id'] .'60') ) {

		if (count($utd_config['plugins']) == 0) {
			// Wrapper (Only display Window, when outdated Plugins was found)
			utd_checkForOutdatedPlugins();
		}

		// Get the wished Page
		$page = intval( str_replace($utd_config['manialink_id'], '', abs($answer[2])) - 10 );
		$xml .= utd_buildPluginListWindow($page, true);

	}
	else if ( ($answer[2] >= (int)$utd_config['manialink_id'] .'10') && ($answer[2] <= (int)$utd_config['manialink_id'] .'60') ) {

		if (count($utd_config['plugins']) == 0) {
			// Wrapper (Only display Window, when outdated Plugins was found)
			utd_checkForOutdatedPlugins();
		}

		// Get the wished Page
		$page = intval( str_replace($utd_config['manialink_id'], '', $answer[2]) - 10 );
		$xml .= utd_buildPluginListWindow($page, true);

	}

	if ($xml != '') {
		// Send to given Player
		$aseco->client->query('SendDisplayManialinkPageToLogin', $player->login, $xml, 0, false);
	}
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

function utd_onEndMap1 ($aseco, $race) {
	global $utd_config;


	if ($utd_config['display_state'] == true) {

		// Close all Windows
		$xml = utd_closeAllWindows();

		// Send to all Players (not only MasterAdmins
		$aseco->client->query('SendDisplayManialinkPage', $xml, 0, false);
	}

	// Empty current list, to refresh them next time
	$utd_config['plugins'] = array();
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

function utd_checkForOutdatedPlugins () {
	global $aseco, $utd_config;


	if ( !isset($aseco->plugin_versions) ) {
		$aseco->console('[plugin.third_party_plugins_uptodate.php] No supported Plugins found, skip checks!');
		return null;
	}

	$result = file_get_contents($utd_config['URL'][0], null, $utd_config['stream_context']);
	if ($result !== false) {

		if (!$releases = $aseco->xml_parser->parseXML($result, false)) {
			$aseco->console('[plugin.third_party_plugins_uptodate.php] Could not parse response from server!');
		}
		else {
			// Generate the checksum for compare
			foreach ($aseco->plugins as $filename) {
				foreach ($aseco->plugin_versions as $id => $installed_plugin) {
					if ($filename == $installed_plugin['plugin']) {
						$plugin = file_get_contents('plugins/'.$filename);
						$plugin = preg_replace('/\s*/', '', $plugin);
						$aseco->plugin_versions[$id]['checksum'] = md5($plugin);
					}
				}
			}

			$releases = $releases['RELEASES']['PLUGIN'];

			$utd_config['plugins'] = array();
			$outdated = false;
			foreach ($releases as $plugin) {
				foreach ($aseco->plugin_versions as $installed_plugin) {
					if ( ($plugin['NAME'][0] == $installed_plugin['plugin']) && ($plugin['AUTHOR'][0] == $installed_plugin['author']) ) {
						if ( version_compare($plugin['RELEASE'][0]['VERSION'][0], $installed_plugin['version'], '>') ) {
							$plugin['UPDATED'] = true;
							$outdated = true;
						}
						else if ( (version_compare($plugin['RELEASE'][0]['VERSION'][0], $installed_plugin['version'], '==')) && ($plugin['RELEASE'][0]['CHECKSUM'][0] != $installed_plugin['checksum']) ) {
							if ($utd_config['ENABLE_CHECKSUM_CHECK'][0] == true) {
								$plugin['UPDATED'] = true;
								$outdated = true;
							}
						}
						else {
							$plugin['UPDATED'] = false;
						}
						$plugin['CURRENT'] = $installed_plugin['version'];

						$utd_config['plugins'][] = $plugin;
					}
				}
			}

			// Now resort the Plugins by require update
			$update = array();
			$plugin = array();
			foreach ($utd_config['plugins'] as $key => &$row) {
				$update[$key] = $row['UPDATED'];
				$plugin[$key] = strtolower($row['TITLE'][0]);
			}
			array_multisort($update, SORT_DESC, $plugin, SORT_ASC, $utd_config['plugins']);
			unset($update, $plugin);

			return $outdated;
		}
	}
	else {
		$aseco->console('[plugin.third_party_plugins_uptodate.php] utd_uptodateCheck() connection failed with the message "'. $http_response_header[0] .'" for url "'. $utd_config['URL'][0] .'"');
	}
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

function utd_buildPluginListWindow ($page = 0, $display = true) {
	global $utd_config;


	$xml  = '<?xml version="1.0" encoding="UTF-8"?>';
	$xml .= '<manialinks>';
	$xml .= '<manialink id="'. $utd_config['manialink_id'] .'00">';

	// Set default to false
	$utd_config['display_state'] = false;
	if ($display == true) {

		// Now set to display
		$utd_config['display_state'] = true;


		// Window
		$xml .= '<frame posn="-40.8 30.55 50" id="ThirdPartyWindow">';	// BEGIN: Window Frame
		$xml .= '<quad posn="-0.2 0.2 0.01" sizen="81.8 59" style="Bgs1InRace" substyle="BgTitle2" id="ThirdPartyWindowBody" ScriptEvents="1"/>';
		$xml .= '<quad posn="1.8 -4.1 0.02" sizen="77.7 49.9" bgcolor="0018"/>';

		// Header Line
		$xml .= '<quad posn="-0.6 0.6 0.02" sizen="82.6 6" style="Bgs1InRace" substyle="BgTitle3_3"/>';
		$xml .= '<quad posn="-0.6 0.6 0.03" sizen="82.6 6" style="Bgs1InRace" substyle="BgTitle3_3" id="ThirdPartyWindowTitle" ScriptEvents="1"/>';

		// Header Icon
		$xml .= '<quad posn="1.8 -0.7 0.20" sizen="3.2 3.2" style="Icons128x128_1" substyle="Editor"/>';
		$xml .= '<label posn="5.5 -1.7 0.04" sizen="75.4 0" textsize="2" scale="0.9" textcolor="000F" text="List of supported and installed Plugins"/>';

		// About
		$xml .= '<label posn="9.6 -55.8 0.04" sizen="23 2" halign="center" valign="center" textsize="1" scale="0.7" url="http://www.undef.name/MPAseco/Third-party-Plugins-UpToDate.php" focusareacolor1="0000" focusareacolor2="FFF5" textcolor="000F" text="THIRD-PARTY PLUGINS UP-TO-DATE/'. $utd_config['version'] .'"/>';

		// Close Button
		$xml .= '<frame posn="76.7 -0.15 0">';
		$xml .= '<quad posn="0 0 0.07" sizen="4.5 4.5" action="'. $utd_config['manialink_id'] .'01" style="Icons64x64_1" substyle="ArrowUp"/>';
//		$xml .= '<quad posn="0 0 0.07" sizen="4.5 4.5" ScriptEvents="1" id="ThirdPartyWindowClose" style="Icons64x64_1" substyle="ArrowUp"/>';
		$xml .= '<quad posn="1.2 -1.2 0.08" sizen="2 2" bgcolor="EEEF"/>';
		$xml .= '<quad posn="0.7 -0.7 0.09" sizen="3.1 3.1" style="Icons64x64_1" substyle="Close"/>';
		$xml .= '</frame>';

		// Minimize Button
		$xml .= '<frame posn="73.4 -0.15 0.05">';
		$xml .= '<quad posn="0 0 0.01" sizen="4.5 4.5" style="Icons64x64_1" substyle="ArrowUp" id="ThirdPartyWindowMinimize" ScriptEvents="1"/>';
		$xml .= '<quad posn="1.2 -1.2 0.02" sizen="2 2" bgcolor="EEEF"/>';
		$xml .= '<label posn="2.33 -2.4 0.03" sizen="6 0" halign="center" valign="center" textsize="3" textcolor="000F" text="$O-"/>';
		$xml .= '</frame>';


		// BEGIN: Frame for Legend/Previous/Next Buttons
		$xml .= '<frame posn="55.35 -53.3 0.04">';

		// Legend button
		$xml .= '<quad posn="13.35 -1 0.12" sizen="3 3" action="'. $utd_config['manialink_id'] .'02" style="Icons64x64_1" substyle="Maximize"/>';
		$xml .= '<quad posn="13.75 -1.4 0.13" sizen="2.1 2.1" bgcolor="000F"/>';
		$xml .= '<quad posn="13.95 -1.6 0.14" sizen="1.8 1.8" style="Icons64x64_1" substyle="TrackInfo"/>';

		// Previous button
		if ($page > 0) {
			$xml .= '<quad posn="16.65 -1 0.12" sizen="3 3" action="-'. $utd_config['manialink_id'] . sprintf("%02d", ($page + 9)) .'" style="Icons64x64_1" substyle="Maximize"/>';
			$xml .= '<quad posn="17.05 -1.4 0.13" sizen="2.1 2.1" bgcolor="000F"/>';
			$xml .= '<quad posn="16.85 -1.2 0.14" sizen="2.5 2.5" style="Icons128x128_1" substyle="Back"/>';
		}
		else {
			$xml .= '<quad posn="16.65 -1.15 0.12" sizen="2.8 2.7" style="UIConstructionSimple_Buttons" substyle="Item"/>';
		}

		// Next button (display only if more pages to display)
		if ( ($page < 50) && (count($utd_config['plugins']) > 18) && (($page+1) < (ceil(count($utd_config['plugins'])/18))) ) {
			$xml .= '<quad posn="19.95 -1 0.12" sizen="3 3" action="'. $utd_config['manialink_id'] . sprintf("%02d", ($page + 11)) .'" style="Icons64x64_1" substyle="Maximize"/>';
			$xml .= '<quad posn="20.35 -1.4 0.13" sizen="2.1 2.1" bgcolor="000F"/>';
			$xml .= '<quad posn="20.15 -1.2 0.14" sizen="2.5 2.5" style="Icons64x64_1" substyle="MediaPlay"/>';
		}
		else {
			$xml .= '<quad posn="19.95 -1.15 0.12" sizen="2.8 2.7" style="UIConstructionSimple_Buttons" substyle="Item"/>';
		}
		$xml .= '</frame>';
		// END: Frame for Previous/Next Buttons



		// BEGIN: Titles
		$xml .= '<frame posn="2.6 -5.4 0">';
		$xml .= '<label posn="0.5 0 0.14" sizen="25 0" textsize="1" textcolor="FF0F" text="Plugin Name"/>';
		$xml .= '<label posn="26 0 0.14" sizen="13.5 0" textsize="1" textcolor="FF0F" text="Author"/>';
		$xml .= '<label posn="45 0 0.14" sizen="5 0" textsize="1" halign="right" textcolor="FF0F" text="Installed"/>';
		$xml .= '<label posn="51 0 0.14" sizen="5 0" textsize="1" halign="right" textcolor="FF0F" text="Released"/>';
		$xml .= '<label posn="52 0 0.14" sizen="8 0" textsize="1" textcolor="FF0F" text="Release Date"/>';
		$xml .= '</frame>';
		// END: Titles

		// BEGIN: Entries
		$xml .= '<frame posn="2.6 -7.6 0">';
		$line_height = 2.43;
		$position = 0;
		for ($i = ($page * 18); $i < (($page * 18) + 18); $i ++) {

			// Is there a Plugin?
			if ( !isset($utd_config['plugins'][$i]) ) {
				break;
			}

			// Get Plugin
			$plugin = $utd_config['plugins'][$i];

			// Set the right Background-Color for a line
			$background = '888E';
			if ($plugin['UPDATED'] == true) {
				if ($plugin['RELEASE'][0]['STATE'][0] == 'update') {
					$background = '3E3E';
				}
				else if ($plugin['RELEASE'][0]['STATE'][0] == 'beta') {
					$background = '09FE';
				}
				else if ($plugin['RELEASE'][0]['STATE'][0] == 'bugfix') {
					$background = 'EE3E';
				}
				else if ($plugin['RELEASE'][0]['STATE'][0] == 'hotfix') {
					$background = 'E33E';
				}
			}

			$xml .= '<label posn="0 '. $position .' 0.12" sizen="76 '. $line_height .'" action="'. $utd_config['manialink_id'] .'02" focusareacolor1="'. $background .'" focusareacolor2="BBBE" text=" "/>';
			$xml .= '<quad posn="-2.5 '. $position .' 0.13" sizen="81 '. $line_height .'" style="BgsPlayerCard" substyle="BgRacePlayerLine"/>';

			$xml .= '<format textsize="1" textcolor="'. (($plugin['UPDATED'] == true) ? '000F' : 'FFFF') .'"/>';
			$xml .= '<label posn="0.5 '. ($position - 0.5) .' 0.14" sizen="25 0" text="'. $plugin['TITLE'][0] .'"/>';
			$xml .= '<label posn="26 '. ($position - 0.5) .' 0.14" sizen="13.5 0" text="'. $plugin['AUTHOR'][0] .'"/>';
			$xml .= '<label posn="45 '. ($position - 0.5) .' 0.14" sizen="5 0" halign="right" text="'. $plugin['CURRENT'] .'"/>';
			$xml .= '<label posn="51 '. ($position - 0.5) .' 0.14" sizen="5 0" halign="right" text="'. $plugin['RELEASE'][0]['VERSION'][0] .'"/>';
			$xml .= '<label posn="52 '. ($position - 0.5) .' 0.14" sizen="8 0" text="'. $plugin['RELEASE'][0]['DATE'][0] .'"/>';

			$xml .= '<quad posn="59.5 '. ($position - 0.12) .' 0.14" sizen="8 2.2" style="Bgs1InRace" substyle="BgButtonBig" url="'. $plugin['URL'][0]['INFORMATION'][0] .'"/>';
			$xml .= '<label posn="63.5 '. ($position - 0.6) .' 0.15" sizen="8 0" halign="center" textsize="1" scale="0.85" text="$0BF$OInformation"/>';

			$xml .= '<quad posn="67.5 '. ($position - 0.12) .' 0.14" sizen="8 2.2" style="Bgs1InRace" substyle="BgButtonBig" url="'. $plugin['URL'][0]['DOWNLOAD'][0] .'"/>';
			$xml .= '<label posn="71.5 '. ($position - 0.6) .' 0.15" sizen="8 0" halign="center" textsize="1" scale="0.85" text="$0BF$ODownload"/>';


			$position -= ($line_height + 0.1);
		}
		$xml .= '</frame>';
		// END: Entries


		$xml .= '</frame>';	// Window

$maniascript = <<<EOL
<script><!--
Void FadeOut (Text ChildId, Real EndOpacity) {
	declare CMlControl Container <=> (Page.GetFirstChild(ChildId) as CMlFrame);

//	2013-05-05: Not possible, because "Opacity" is not accessible with ManiaScript yet
//	while (Container.Opacity > EndOpacity) {
//		Container.Opacity -= 0.05;
//		yield;
//	}
	Container.Unload();
}
Void WipeOut (Text ChildId) {
	declare CMlControl Container <=> (Page.GetFirstChild(ChildId) as CMlFrame);
	declare Real EndPosnX = 0.0;
	declare Real EndPosnY = 0.0;
	declare Real PosnDistanceX = (EndPosnX - Container.PosnX);
	declare Real PosnDistanceY = (EndPosnY - Container.PosnY);

	while (Container.RelativeScale > 0.0) {
		Container.PosnX += (PosnDistanceX / 20);
		Container.PosnY += (PosnDistanceY / 20);
		Container.RelativeScale -= 0.05;
		yield;
	}
	Container.Unload();
}
Void Minimize (Text ChildId) {
	declare CMlControl Container <=> (Page.GetFirstChild(ChildId) as CMlFrame);
	declare Real EndPosnX = (-40.8 * 2.5);
	declare Real EndPosnY = (30.55 * 1.875);
	declare Real PosnDistanceX = (EndPosnX - Container.PosnX);
	declare Real PosnDistanceY = (EndPosnY - Container.PosnY);

	while (Container.RelativeScale > 0.2) {
		Container.PosnX += (PosnDistanceX / 16);
		Container.PosnY += (PosnDistanceY / 16);
		Container.RelativeScale -= 0.05;
		yield;
	}
}
Void Maximize (Text ChildId) {
	declare CMlControl Container <=> (Page.GetFirstChild(ChildId) as CMlFrame);
	declare Real EndPosnX = (-40.8 * 2.5);
	declare Real EndPosnY = (30.55 * 1.875);
	declare Real PosnDistanceX = (EndPosnX - Container.PosnX);
	declare Real PosnDistanceY = (EndPosnY - Container.PosnY);

	while (Container.RelativeScale < 1.0) {
		Container.PosnX += (PosnDistanceX / 16);
		Container.PosnY += (PosnDistanceY / 16);
		Container.RelativeScale += 0.05;
		yield;
	}
}
main () {
	declare CMlControl Container <=> (Page.GetFirstChild("ThirdPartyWindow") as CMlFrame);
	declare CMlQuad Quad;
	declare Boolean MoveWindow = False;
	declare Boolean IsMinimized = False;
	declare Real MouseDistanceX = 0.0;
	declare Real MouseDistanceY = 0.0;

	while (True) {
		if (MoveWindow == True) {
			Container.PosnX = (MouseDistanceX + MouseX);
			Container.PosnY = (MouseDistanceY + MouseY);
		}
		if (MouseLeftButton == True) {
			foreach (Event in PendingEvents) {
				if (Event.ControlId == "ThirdPartyWindowTitle") {
					MouseDistanceX = (Container.PosnX - MouseX);
					MouseDistanceY = (Container.PosnY - MouseY);
					MoveWindow = True;
				}
			}
		}
		else {
			MoveWindow = False;
		}
		foreach (Event in PendingEvents) {
			switch (Event.Type) {
				case CMlEvent::Type::MouseClick : {
					if (Event.ControlId == "ThirdPartyWindowClose") {
						WipeOut("ThirdPartyWindow");
					}
					else if ( (Event.ControlId == "ThirdPartyWindowMinimize") && (IsMinimized == False) ) {
						Minimize("ThirdPartyWindow");
						IsMinimized = True;
					}
					else if ( (Event.ControlId == "ThirdPartyWindowBody") && (IsMinimized == True) ) {
						Maximize("ThirdPartyWindow");
						IsMinimized = False;
					}
				}
			}
		}
		yield;
	}
}
--></script>
EOL;
		$xml .= $maniascript;

	}

	$xml .= '</manialink>';
	$xml .= '</manialinks>';

	return $xml;
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

function utd_buildLegendWindow ($display = true) {
	global $utd_config;


	$xml =  '<?xml version="1.0" encoding="UTF-8"?>';
	$xml .= '<manialinks>';
	$xml .= '<manialink id="'. $utd_config['manialink_id'] .'01">';

	if ($display == true) {
		$xml .= '<frame posn="-20.9 15.9 51">';	// BEGIN: Window Frame
		$xml .= '<quad posn="-0.2 0.2 0.01" sizen="41.7 32.05" style="Bgs1InRace" substyle="BgTitle2"/>';
		$xml .= '<quad posn="1.8 -4.1 0.02" sizen="37.6 25.1" bgcolor="0018"/>';

		// Header Line
		$xml .= '<quad posn="-0.6 0.6 0.02" sizen="42.5 6" style="Bgs1InRace" substyle="BgTitle3_3"/>';
		$xml .= '<quad posn="-0.6 0.6 0.03" sizen="42.5 6" style="Bgs1InRace" substyle="BgTitle3_3"/>';

		// Title
		$xml .= '<quad posn="1.4 -1 0.04" sizen="3 3" style="UIConstructionSimple_Buttons" substyle="Tools"/>';
		$xml .= '<label posn="4.9 -1.6 0.04" sizen="37 0" textsize="2" scale="0.9" textcolor="000F" text="Legend"/>';

		// Close Button
		$xml .= '<frame posn="36.6 -0.15 0.05">';
		$xml .= '<quad posn="0 0 0.01" sizen="4.5 4.5" action="'. $utd_config['manialink_id'] .'03" style="Icons64x64_1" substyle="ArrowUp"/>';
		$xml .= '<quad posn="1.2 -1.2 0.02" sizen="2 2" bgcolor="EEEF"/>';
		$xml .= '<quad posn="0.7 -0.7 0.03" sizen="3.1 3.1" style="Icons64x64_1" substyle="Close"/>';
		$xml .= '</frame>';


		// Frame for the Legend
		$xml .= '<frame posn="-4 -5 0">';

		$xml .= '<quad posn="8.8 -2.2 0.12" sizen="11 3" bgcolor="E33E"/>';
		$xml .= '<label posn="14.2 -3.05 0.12" sizen="11 3" halign="center" textsize="1" scale="0.9" text="$000Critical Update"/>';
		$xml .= '<label posn="22 -3.05 0.12" sizen="35 3" textsize="1" scale="0.9" text="$FFFUpdate as soon as possible!"/>';

		$xml .= '<quad posn="8.8 -6.2 0.12" sizen="11 3" bgcolor="EE3E"/>';
		$xml .= '<label posn="14.2 -7.05 0.12" sizen="11 3" halign="center" textsize="1" scale="0.9" text="$000Bugfix Update"/>';
		$xml .= '<label posn="22 -7.05 0.12" sizen="35 3" textsize="1" scale="0.9" text="$FFFUpdate is required!"/>';

		$xml .= '<quad posn="8.8 -10.2 0.12" sizen="11 3" bgcolor="3E3E"/>';
		$xml .= '<label posn="14.2 -11.05 0.12" sizen="11 3" halign="center" textsize="1" scale="0.9" text="$000Feature Update"/>';
		$xml .= '<label posn="22 -11.05 0.12" sizen="35 3" textsize="1" scale="0.9" text="$FFFUpdate if you want the new features."/>';

		$xml .= '<quad posn="8.8 -14.2 0.12" sizen="11 3" bgcolor="09FE"/>';
		$xml .= '<label posn="14.2 -15.05 0.12" sizen="11 3" halign="center" textsize="1" scale="0.9" text="$000Beta Release"/>';
		$xml .= '<label posn="22 -15.05 0.12" sizen="35 3" textsize="1" scale="0.9" text="$FFFUpdate, that can run unstable."/>';

		$xml .= '<quad posn="8.8 -18.2 0.12" sizen="11 3" bgcolor="888E"/>';
		$xml .= '<label posn="14.2 -19.05 0.12" sizen="11 3" halign="center" textsize="1" scale="0.9" text="$FFFUp-To-Date"/>';
		$xml .= '<label posn="22 -19.05 0.12" sizen="35 3" textsize="1" scale="0.9" text="$FFFInstalled version is up-to-date."/>';

		$xml .= '</frame>';


		$xml .= '</frame>';	// END: Window Frame
	}

	$xml .= '</manialink>';
	$xml .= '</manialinks>';

	return $xml;
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

function utd_closeAllWindows () {
	global $utd_config;


	$xml =  '<?xml version="1.0" encoding="UTF-8"?>';
	$xml .= '<manialinks>';
	$xml .= '<manialink id="'. $utd_config['manialink_id'] .'00"></manialink>';	// Window
	$xml .= '<manialink id="'. $utd_config['manialink_id'] .'01"></manialink>';	// LegendWindow
	$xml .= '</manialinks>';

	return $xml;
}

?>
